<?php

namespace Drupal\sdc_styleguide\Service;

use Drupal\Core\Serialization\Yaml;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\Theme\ComponentPluginManager;
use Symfony\Component\Finder\Finder;

/**
 * Class to manage demos.
 */
class SDCDemoManager {
  use StringTranslationTrait;

  /**
   * @var array
   * The list of demos:
   *  - index: All demos accessible by name.
   *  - groups: All demos accessible by SDC group.
   */
  private $demos = [];

  /**
   * The controller constructor.
   */
  public function __construct(
    private readonly ComponentPluginManager $pluginManagerSdc,
  ) {}

  /**
   * @param $id
   *
   * @return mixed|null
   */
  public function getDemoById($id) {
    if (empty($this->demos)) {
      $this->getDemos();
    }
    return $this->demos['index'][$id] ?? NULL;
  }

  /**
   *  Gets a list of all the available SDC on site and their corresponding demos
   *  if available.
   *
   * @return array
   *  The list of demos grouped by Group and Component.
   */
  public function getDemos() {
    if (!empty($this->demos)) {
      return $this->demos['groups'];
    }

    // Avoids having to regenerate all demos in the same request.
    // @TODO: Maybe add cache?
    $this->demos = [
      'groups' => [],
      'index' => [],
    ];

    // Builds demos.
    $componentDemos = &$this->demos['groups'];
    $ungroupedIndex = $this->t('Ungrouped')->render();
    foreach ($this->pluginManagerSdc->getAllComponents() as $component) {
      $definition = $component->getPluginDefinition();
      $group = $definition['group'] ?? $ungroupedIndex;
      $componentId = $component->getPluginId();

      if (!isset($componentDemos[$group])) {
        $componentDemos[$group] = [];
      }

      // Adds component.
      $componentDemos[$group][$componentId] = [
        'name' => $definition['name'],
        'demos' => [],
      ];
      $demos = &$componentDemos[$group][$componentId]['demos'];

      // Finds the demos for the current component and adds them to the explorer.
      $finder = new Finder();
      $component_name = $definition['machineName'];
      $finder->in($definition['path'])->files()->name("{$component_name}.demo.*.yml");
      foreach ($finder as $file) {
        $key = str_replace(['.yml', "{$component_name}.demo."], '', $file->getFilename());
        $contents = $file->getContents();
        $demo_data = Yaml::decode($contents);
        $demo_data['component_id'] = $componentId;
        $demos[$key] = $demo_data;
        $this->demos['index']["{$componentId}.demo.{$key}"] = $demo_data;
      }

      // Sorts by component readable name.
      uasort($demos, fn ($a, $b) => strcmp($a['name'], $b['name']));
    }

    foreach ($componentDemos as &$group) {
      uasort($group, fn($a, $b) => strcmp($a['name'], $b['name']));
    }
    ksort($componentDemos);

    return $componentDemos;
  }


}
